#include <iostream>
#include <fstream>
#include <opencv2/opencv.hpp>
#include <opencv2/saliency.hpp>
#include <boost/filesystem.hpp>
#include <boost/filesystem/path.hpp>

using namespace std;
using namespace cv;
using namespace saliency;

static void help()
{
    cout << "Usage: ./bing <input_image_name> <trained_model_dir> <output_file_name>"
        << endl;
}

// 抽出された領域のx座標の最小値、y座標の最小値と
// x座標の最大値、y座標の最大値ををoutputFileNameに書き込む関数
void outputBoundingBox(const vector<Vec4i>& saliencyMap,
        const string inputImageName,
        const string outputFileName){
    const int TOP_N = 10000;
    int count = 0;
    
    //入力画像の絶対パスを取得
    boost::filesystem::path filename(inputImageName);
    const String input_image_abs_path = boost::filesystem::absolute(filename).string();

    ofstream ofs;
    ofs.open(outputFileName.c_str());
    
    //1行目にカラム名を書き込み
    ofs << "filename,xmin,ymin,xmax,ymax" << endl;
    
    //1領域に対し、1行ずつ書き出し
    for(vector<Vec4i>::const_iterator it = saliencyMap.begin();
            it != saliencyMap.end(); ++it){
        ofs << input_image_abs_path << ',' << (*it)[0] << ',' << (*it)[1] << ','
            << (*it)[2] << ',' << (*it)[3] << endl;
        if(TOP_N <= count++){
            break; 
        }
    }
    ofs.close();
}

int main(int argc, char** argv){

    if(argc != 4){ 
        help();
        return -1;
    }

    string input_image_name = argv[1];
    string trained_model_dir = argv[2];
    string output_file_name = argv[3];
    
    //BINGで候補領域を検出
    Mat src = imread(input_image_name);
    Ptr<Saliency> saliencyAlgorithm = Saliency::create("BING");
    vector<Vec4i> saliencyMap;
    saliencyAlgorithm.dynamicCast<ObjectnessBING>()->setTrainingPath(
            trained_model_dir);
    if(saliencyAlgorithm->computeSaliency(src, saliencyMap)){ 
        std::cout << "Objectness done" << std::endl;
    }
    
    outputBoundingBox(saliencyMap, input_image_name, output_file_name);

    return 0;
}
