#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import os

# 各候補領域の確信度データを読み込み
df = pd.read_hdf('output.h5', 'df')
print(df.shape)
print(df.iloc[0])

# 環境変数$CAFFE_DIRを読み込み
caffe_root = os.environ.get("CAFFE_DIR")

# synsetデータをlabels_dfに格納
with open(caffe_root + '/data/ilsvrc12/det_synset_words.txt') as f:
    labels_df = pd.DataFrame([
        {
            'synset_id': l.strip().split(' ')[0],
            'name': ' '.join(l.strip().split(' ')[1:]).split(',')[0]
        }
        for l in f.readlines()
    ])
labels_df.sort('synset_id')
predictions_df = pd.DataFrame(
    np.vstack(df.prediction.values), columns=labels_df['name'])
print(predictions_df.iloc[0])

# 上位10クラスを表示
max_s = predictions_df.max(0)
max_s.sort(ascending=False)
print(max_s[:10])

# personとbicycleにおいて確信度が最大とするインデックスを探索
i = predictions_df['person'].argmax()
j = predictions_df['bicycle'].argmax()

# 確信度1位の検出領域における上位5クラスを表示
f = pd.Series(df['prediction'].iloc[i], index=labels_df['name'])
print('Top detection:')
print(f.order(ascending=False)[:5])
print('')

# 確信度2位の検出領域における上位5クラスを表示
f = pd.Series(df['prediction'].iloc[j], index=labels_df['name'])
print('Second-best detection:')
print(f.order(ascending=False)[:5])

# 確信度1位を赤、2位を青の矩形で表示
im = plt.imread('fish-bike.jpg')
plt.imshow(im)
currentAxis = plt.gca()

det = df.iloc[i]
coords = (det['xmin'], det['ymin']), det['xmax'] - \
    det['xmin'], det['ymax'] - det['ymin']
currentAxis.add_patch(
    plt.Rectangle(*coords, fill=False, edgecolor='r', linewidth=5))

det = df.iloc[j]
coords = (det['xmin'], det['ymin']), det['xmax'] - \
    det['xmin'], det['ymax'] - det['ymin']
currentAxis.add_patch(
    plt.Rectangle(*coords, fill=False, edgecolor='b', linewidth=5))

plt.show()
