#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
from PIL import Image
import os
import caffe

# 環境変数$CAFFE_DIRを取得
caffe_root = os.environ.get("CAFFE_DIR")

# プロット設定
plt.rcParams['figure.figsize'] = (10, 10)
plt.rcParams['image.interpolation'] = 'nearest'
plt.rcParams['image.cmap'] = 'gray'

# CPUモードに設定
caffe.set_mode_cpu()

# netを読み込み、どのようなデータとパラメータがあるのかを出力します
net = caffe.Net(caffe_root + '/examples/net_surgery/conv.prototxt', caffe.TEST)
print("blobs {}\nparams {}".format(net.blobs.keys(), net.params.keys()))

# CaffeNetの読み込み
net = caffe.Net(caffe_root + '/models/bvlc_reference_caffenet/deploy.prototxt', 
                caffe_root +
                '/models/bvlc_reference_caffenet/bvlc_reference_caffenet.caffemodel', 
                caffe.TEST)

# netにおける全結合層のパラメータの抽出
params = ['fc6', 'fc7', 'fc8']
# fc_params = {name: (weights, biases)} という形式
fc_params = {
    pr: (net.params[pr][0].data, net.params[pr][1].data) for pr in params}

# fc6, fc7, fc8の各重みとバイアスのデータ構造を出力･･･(1)
for fc in params:
    print '{} weights are {} dimensional and biases are {} dimensional'.format(
        fc, fc_params[fc][0].shape, fc_params[fc][1].shape)

# パラメータを移植するためのFCNの読み込み
net_full_conv = caffe.Net(caffe_root + '/examples/net_surgery/bvlc_caffenet_full_conv.prototxt', 
                          caffe_root +
                          '/models/bvlc_reference_caffenet/bvlc_reference_caffenet.caffemodel',
                          caffe.TEST)
params_full_conv = ['fc6-conv', 'fc7-conv', 'fc8-conv']
# conv_params = {name: (weights, biases)} という形式
conv_params = {pr: (net_full_conv.params[pr][0].data, net_full_conv.params[
                    pr][1].data) for pr in params_full_conv}

# fc6-conv, fc7-conv, fc8-convの各重みとバイアスのデータ構造を出力･･･(2)
for conv in params_full_conv:
    print '{} weights are {} dimensional and biases are {} dimensional'.format(
        conv, conv_params[conv][0].shape, conv_params[conv][1].shape)

# CaffeNetの全結合層の重みをFCNに再配置する作業
for pr, pr_conv in zip(params, params_full_conv):
    conv_params[pr_conv][0].flat = fc_params[pr][0].flat
    conv_params[pr_conv][1][...] = fc_params[pr][1]

# net_full_convをモデルとして保存
net_full_conv.save(
    caffe_root + '/examples/net_surgery/bvlc_caffenet_full_conv.caffemodel')

# 入力画像を読み込み後、前処理
im = caffe.io.load_image(caffe_root + '/examples/images/cat.jpg')
transformer = caffe.io.Transformer(
    {'data': net_full_conv.blobs['data'].data.shape})
transformer.set_mean('data', np.load(
    caffe_root + '/python/caffe/imagenet/ilsvrc_2012_mean.npy').mean(1).mean(1))
transformer.set_transpose('data', (2, 0, 1))
transformer.set_channel_swap('data', (2, 1, 0))
transformer.set_raw_scale('data', 255.0)

# それぞれのウィンドウ(227x227)において分類されたラベルを出力･･･(3)
out = net_full_conv.forward_all(
    data=np.asarray([transformer.preprocess('data', im)]))
print out['prob'][0].argmax(axis=0)

# 入力画像とそれぞれの位置におけるトラネコ(281)の確信度マップを可視化
plt.subplot(1, 2, 1)
plt.imshow(transformer.deprocess('data', net_full_conv.blobs['data'].data[0]))
plt.subplot(1, 2, 2)
plt.imshow(out['prob'][0, 281])
plt.show()
