#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
import os
import caffe

# pltの設定を変更します
# 画像サイズを10inch x 10inchに指定します
plt.rcParams['figure.figsize'] = (10, 10)

# 画像の補間処理を無効化します
plt.rcParams['image.interpolation'] = 'none'

# Color mapを明示します
plt.rcParams['image.cmap'] = 'gray'

# 環境変数$CAFFE_DIRを取得
caffe_root = os.environ.get("CAFFE_DIR")

# CPUモードにセット
caffe.set_mode_cpu()

# 学習済みモデルの読み込み
net = caffe.Net(caffe_root + '/models/bvlc_alexnet/deploy.prototxt',
                caffe_root + '/models/bvlc_alexnet/bvlc_alexnet.caffemodel',
                caffe.TEST)

# dataを可視化する関数
def vis_square(data, padsize=1, padval=0):

    # 値が0～1の範囲に収まるように正規化します
    data -= data.min()
    data /= data.max()
    
    # データ数(この場合はカーネル数)を基に可視化する画像が正方形になるように計算します
    n = int(np.ceil(np.sqrt(data.shape[0])))
    padding = ((0, n ** 2 - data.shape[0]),
               (0, padsize),
               (0, padsize)) + ((0, 0),
                                ) * (data.ndim - 3)
    data = np.pad(
        data, padding, mode='constant', constant_values=(padval, padval))
    
    # 各データ(この場合はカーネル)をタイル状に整列させます
    data = data.reshape(
        (n, n) + data.shape[1:]).transpose((0, 2, 1, 3) + tuple(range(4, data.ndim + 1)))
    data = data.reshape(
        (n * data.shape[1], n * data.shape[3]) + data.shape[4:])
    
    # 表示処理です
    plt.imshow(data)
    plt.show()

# net.paramsの形式を確認しています。
print("net.paramsのkeyとvalueのリストを表示")
print([(k, v[0].data.shape) for k, v in net.params.items()])

# conv1のカーネルを可視化します
# パラメータは[重みデータ, バイアスデータ]のリスト形式になっています
filters = net.params['conv1'][0].data
vis_square(filters.transpose(0, 2, 3, 1))

# conv2のカーネルを可視化します
filters = net.params['conv2'][0].data
vis_square(filters[:48].reshape(48**2, 5, 5))
